function [Dataset numberOfSubDirs, larvaPerSubDir] = readDataForAnalysis(rootPath)

Dataset = dataset();
larvaID = 0;
larvaPerSubDir = [];

%% get all CSV-Files
[CSVFiles numberOfSubDirs] = getAllFiles(rootPath);

%%
for i=1:size(CSVFiles,1)
    %% Initialize variables.
    filename = CSVFiles{i};
    delimiter = ',';
    startRow = 2;
    
    %% Open the text file.
    fileID = fopen(filename,'r');
    lineIjustReadIn = fgetl(fileID);
    numColumns = sum(lineIjustReadIn == delimiter);
    larvaPerSubDir = [larvaPerSubDir numColumns];
    fseek(fileID, 0, -1);
    
    %% Format string for each line of text:
    formatSpec = ['%s' repmat('%f', 1, numColumns) '%[^\n\r]'];
    
    %% Read columns of data according to format string.
    % This call is based on the structure of the file used to generate this
    % code. If an error occurs for a different file, try regenerating the code
    % from the Import Tool.
    dataArray = textscan(fileID, formatSpec, 'Delimiter', delimiter, 'EmptyValue', NaN, 'HeaderLines', startRow-1, 'ReturnOnError', false);
    
    %% Close the text file.
    fclose(fileID);
    
    %% Post processing for unimportable data.
    % No unimportable data rules were applied during the import, so no post
    % processing code is included. To generate code which works for
    % unimportable data, select unimportable cells in a file and regenerate the
    % script.
    
    cellLarva       = repmat({'larva'}, 1, numColumns);
    cellLarvaID     = cell(1, numColumns);
    for i=1:numColumns
        cellLarvaID{i} = larvaID;
        larvaID = larvaID + 1;
    end
    cellLarvaIDFinal    = cellfun(@(x,y) [x num2str(y)], cellLarva, cellLarvaID, 'uniformoutput', 0);
    cellLarvaIDFinal    = ['ID' cellLarvaIDFinal];
    
    %% Create output variable
    Data = dataset(dataArray{1:end-1}, 'VarNames', cellLarvaIDFinal);
    if(isempty(Dataset))
        Dataset = Data;
    else
        % IMPORTANT: join(X,Y,'ID') needs an appropriate ID entry in X
        % (e.g. 'mom_x(1001)') so that X must be the dataset with more
        % entries (i.e. with more time steps). This is guaranteed by
        % the following if-condition. However, if join(Y,X,'ID') must
        % be used (else case), the larval variable names are not in
        % correct order anymore! This is fixed by the sort section
        % below...
        
        
        Dataset = unifyData(Data, Dataset);
        
    end
end

%% remove animals containing only NaNs
% since all dataset entries (i.e. larvae) are cutted so that they have the
% same length, some animals from very long movies only exist in the frames
% which was cutted. The result is an animal in the dataset which consists
% of NaNs only (all meaningful entries are cut away). These animals need to
% be removed since there is no useful data in them.
% larva_names = Dataset.Properties.VarNames(2:end);
% collision_indicator = selectFeature(Dataset,'in_collision');
% larva_names_to_delete = cell(1,0);
% for k=1:numel(larva_names)
%     % get collision indicator for the larvae
%     l_collision = collision_indicator.(larva_names{k});
%     
%     % find first frame number in which the larva exists
%     is_not_nan_idx = find(~isnan(l_collision)==1);
%     
%     if(numel(is_not_nan_idx) == 0)
%        larva_names_to_delete{1,end+1}=larva_names{k};
%     end
% end
% 
% for k=1:numel(larva_names_to_delete)
%    Dataset.(larva_names_to_delete{k}) = []; 
% end


% %% Sort columns in talbe ('larva0', 'larva1', 'larva2', ...)
% 
% % larva names (e.g. 'larva101') might not be in order anymore
% % get the larva names ('ID', 'larva101', 'larva1334', ...)
% larva_names = Dataset.Properties.VarNames;
% % get the numbers from the larva names (101, 1334, ...)
% larva_numbers = cellfun(@str2num, cellfun(@(x) [regexprep(x,'larva','')], larva_names(2:end), 'UniformOutput', false));
% % sort the numbers and save the permutation (isort)
% [~,isort]=sort(larva_numbers);
% % add a 0 to the permutation and increment all values (because of the
% % 'ID' variable name which must be in the first column
% adjusted_isort = [0,isort]+1;
% % reorder the dataset based on the permutation
% Dataset = Dataset(:,adjusted_isort);
%% Clear temporary variables
clearvars filename delimiter startRow formatSpec fileID dataArray ans Data CSVFiles cellLarva cellLarvaID cellLarvaIDFinal larvaID;
end

function key_vec = getKeyNames(Data)
% get the ids (e.g. 'mom_x(10)')
complete_ids = Data.ID;
% remove everything after '(' (including '(') -> 'mom_x'
only_id_names = cellfun(@(x) x(1:strfind(x,'(')-1), complete_ids, 'UniformOutput', false);

% make to cell arrays: A = 'mom_x', 'mom_x', ..., 'mom_y', 'dummy' 
% and                  B = 'dummy', 'mom_x', ..., 'mom_y', 'mom_y'
only_id_names_shifted = only_id_names;
only_id_names_shifted(2:end+1) = only_id_names(1:end);
only_id_names_shifted(1) = {'dummy'};
only_id_names(end+1) = {'dummy'};

% find differences between A and B (differences will be detected at the
% 'dummy' positions and when the parameter name changes)
changes = ~strcmp(only_id_names, only_id_names_shifted);

% changes looks like (1,0,0,...,0,1,0,0,...,0,1), whereas teh first and
% last '1' indicates the 'dummy' positions. the other '1' point to a unique
% name in 'only_id_names'. By removing the first '1' from 'changes', all
% remaining '1's will point to a unique name in 'only_id_names'
changes = changes(2:end);
% select only the unique names
key_vec = only_id_names(changes);
end

% function unifies two datasets with two special conditions
% 1. If dataset A has keys which don't exist in dataset B, this key is 
% included into the dataset A (with NaN entries)
% 2. if dataset A has less frames than dataset B, the datasets
% are unified so that the but removing the additional frames from B
function Dataset = unifyData(Data, Dataset)
% get key names and number of frames
Data_key_names = getKeyNames(Data);
Data_num_frames = size(selectFeature(Data, 'mom_x'),1);

Dataset_key_names = getKeyNames(Dataset);
Dataset_num_frames = size(selectFeature(Dataset, 'mom_x'),1);

% add missing keys to 'Dataset' (Data has additional keys)
% get the missing keys in 'Dataset'
missing_keys = setdiff(Data_key_names, Dataset_key_names);
% are there missing keys in 'Dataset'
if numel(missing_keys) > 0
    % add all missing keys...
    for i=1:numel(missing_keys)
        current_key = missing_keys{i};
        % a key name consits of the current_key string (e.g. mom_x) named
        % prefix, the frame number (e.g. 10) named suffix and two brackets 
        % (i.e. '(' and '(').
        % The resultsnt key name is than prefix '(' suffix ')' (e.g.
        % 'mom_x(10)')
        prefix = cell(Dataset_num_frames,1);
        ix = cellfun('isempty',prefix);
        prefix(ix) = {current_key};
        open_bracket = cell(Dataset_num_frames,1);
        open_bracket(ix) = {'('};
        close_bracket = cell(Dataset_num_frames,1);
        close_bracket(ix) = {')'};
        suffix = 0:Dataset_num_frames-1;
        suffix = num2cell(suffix)';
        % new_key stores all key_names (e.g. mom_x(0), mom_x(1), ...) 
        new_key = cellfun(@(x,y,o,c) [x o num2str(y) c], prefix, suffix, open_bracket, close_bracket, 'uniformoutput', 0);
        
        % generate a nan-containing dataset with appropriate NaN entries
        tmp_mat = ones(Dataset_num_frames, size(Dataset,2))*NaN;
        ds = mat2dataset(tmp_mat);
        ds.Properties.VarNames = Dataset.Properties.VarNames;
        ds.ID = new_key;
        % add this key into the Dataset
        Dataset = union(Dataset, ds);
    end
end

% add missing keys to 'Data' (Dataset has additional keys)
% get the missing keys in 'Data'
missing_keys = setdiff(Dataset_key_names, Data_key_names);
% are there missing keys in 'Data'
if numel(missing_keys) > 0
    % add all missing keys...
    for i=1:numel(missing_keys)
        current_key = missing_keys{i};
        % a key name consits of the current_key string (e.g. mom_x) named
        % prefix, the frame number (e.g. 10) named suffix and two brackets 
        % (i.e. '(' and '(').
        % The resultsnt key name is than prefix '(' suffix ')' (e.g.
        % 'mom_x(10)')
        prefix = cell(Data_num_frames,1);
        ix = cellfun('isempty',prefix);
        prefix(ix) = {current_key};
        open_bracket = cell(Data_num_frames,1);
        open_bracket(ix) = {'('};
        close_bracket = cell(Data_num_frames,1);
        close_bracket(ix) = {')'};
        suffix = 0:Data_num_frames-1;
        suffix = num2cell(suffix)';
        % new_key stores all key_names (e.g. mom_x(0), mom_x(1), ...) 
        new_key = cellfun(@(x,y,o,c) [x o num2str(y) c], prefix, suffix, open_bracket, close_bracket, 'uniformoutput', 0);
        
        % generate a nan-containing dataset with appropriate NaN entries
        tmp_mat = ones(Data_num_frames, size(Data,2))*NaN;
        ds = mat2dataset(tmp_mat);
        ds.Properties.VarNames = Data.Properties.VarNames;
        ds.ID = new_key;
        % add this key into the Data
        Data = union(Data, ds);
        
    end
end

% check number of frames...
if (Data_num_frames >= Dataset_num_frames)
    % if Data has more frames than Dataset
    Dataset = join(Dataset, Data, 'ID');
else
    Dataset = join(Data, Dataset, 'ID');
end

end
